@@ -17,12 +17,11 @@
 
 #include "./arrow_types.h"
 
-#if defined(ARROW_R_WITH_ARROW)
-
 #include <arrow/array.h>
 #include <arrow/chunked_array.h>
 #include <arrow/compute/api.h>
 #include <arrow/util/bitmap_reader.h>
+#include <arrow/visit_data_inline.h>
 
 #include <cpp11/altrep.hpp>
 #include <cpp11/declarations.hpp>
@@ -51,6 +50,9 @@ extern "C" {
 
 #include "./r_task_group.h"
 
+// defined in array_to_vector.cpp
+SEXP Array__as_vector(const std::shared_ptr<arrow::Array>& array);
+
 namespace arrow {
 namespace r {
 namespace altrep {
@@ -69,14 +71,40 @@ R_xlen_t Standard_Get_region<int>(SEXP data2, R_xlen_t i, R_xlen_t n, int* buf)
   return INTEGER_GET_REGION(data2, i, n, buf);
 }
 
-void DeleteArray(std::shared_ptr<Array>* ptr) { delete ptr; }
-using Pointer = cpp11::external_pointer<std::shared_ptr<Array>, DeleteArray>;
-
-// the Array that is being wrapped by the altrep object
-static const std::shared_ptr<Array>& GetArray(SEXP alt) {
-  return *Pointer(R_altrep_data1(alt));
+template <typename T>
+void DeletePointer(std::shared_ptr<T>* ptr) {
+  delete ptr;
 }
 
+template <typename T>
+using Pointer = cpp11::external_pointer<std::shared_ptr<T>, DeletePointer<T>>;
+
+// the ChunkedArray that is being wrapped by the altrep object
+const std::shared_ptr<ChunkedArray>& GetChunkedArray(SEXP alt) {
+  return *Pointer<ChunkedArray>(R_altrep_data1(alt));
+}
+
+struct ArrayResolve {
+  ArrayResolve(const std::shared_ptr<ChunkedArray>& chunked_array, int64_t i) {
+    for (int idx_chunk = 0; idx_chunk < chunked_array->num_chunks(); idx_chunk++) {
+      std::shared_ptr<Array> chunk = chunked_array->chunk(idx_chunk);
+      auto chunk_size = chunk->length();
+      if (i < chunk_size) {
+        index_ = i;
+        array_ = chunk;
+        position_ = idx_chunk;
+        break;
+      }
+
+      i -= chunk_size;
+    }
+  }
+
+  std::shared_ptr<Array> array_;
+  int64_t index_ = 0;
+  int64_t position_ = 0;
+};
+
 // base class for all altrep vectors
 //
 // data1: the Array as an external pointer.
@@ -86,9 +114,11 @@ static const std::shared_ptr<Array>& GetArray(SEXP alt) {
 template <typename Impl>
 struct AltrepVectorBase {
   // store the Array as an external pointer in data1, mark as immutable
-  static SEXP Make(const std::shared_ptr<Array>& array) {
-    SEXP alt = R_new_altrep(Impl::class_t, Pointer(new std::shared_ptr<Array>(array)),
-                            R_NilValue);
+  static SEXP Make(const std::shared_ptr<ChunkedArray>& chunked_array) {
+    SEXP alt = R_new_altrep(
+        Impl::class_t,
+        Pointer<ChunkedArray>(new std::shared_ptr<ChunkedArray>(chunked_array)),
+        R_NilValue);
     MARK_NOT_MUTABLE(alt);
 
     return alt;
@@ -96,26 +126,26 @@ struct AltrepVectorBase {
 
   // Is the vector materialized, i.e. does the data2 slot contain a
   // standard R vector with the same data as the array.
-  static bool IsMaterialized(SEXP alt) { return !Rf_isNull(R_altrep_data2(alt)); }
+  static bool IsMaterialized(SEXP alt) { return !Rf_isNull(Impl::Representation(alt)); }
 
-  static R_xlen_t Length(SEXP alt) { return GetArray(alt)->length(); }
+  static R_xlen_t Length(SEXP alt) { return GetChunkedArray(alt)->length(); }
 
-  static int No_NA(SEXP alt) { return GetArray(alt)->null_count() == 0; }
+  static int No_NA(SEXP alt) { return GetChunkedArray(alt)->null_count() == 0; }
 
   static int Is_sorted(SEXP alt) { return UNKNOWN_SORTEDNESS; }
 
   // What gets printed on .Internal(inspect(<the altrep object>))
   static Rboolean Inspect(SEXP alt, int pre, int deep, int pvec,
                           void (*inspect_subtree)(SEXP, int, int, int)) {
-    const auto& array = GetArray(alt);
-    Rprintf("arrow::Array<%s, %d nulls> len=%d, Array=<%p>\n",
-            array->type()->ToString().c_str(), array->null_count(), array->length(),
-            array.get());
+    const auto& chunked_array = GetChunkedArray(alt);
+    Rprintf("arrow::ChunkedArray<%p, %s, %d chunks, %d nulls> len=%d\n",
+            chunked_array.get(), chunked_array->type()->ToString().c_str(),
+            chunked_array->num_chunks(), chunked_array->null_count(),
+            chunked_array->length());
     return TRUE;
   }
 
-  // Duplication is done by first materializing the vector and
-  // then make a lazy duplicate of data2
+  // Materialize and then duplicate data2
   static SEXP Duplicate(SEXP alt, Rboolean /* deep */) {
     return Rf_duplicate(Impl::Materialize(alt));
   }
@@ -127,6 +157,12 @@ struct AltrepVectorBase {
   static SEXP Serialized_state(SEXP alt) { return Impl::Materialize(alt); }
 
   static SEXP Unserialize(SEXP /* class_ */, SEXP state) { return state; }
+
+  // default methods used when data2 is the representation
+  // this is overridden when data2 needs to be richer (e.g. for factors)
+  static SEXP Representation(SEXP alt) { return R_altrep_data2(alt); }
+
+  static void SetRepresentation(SEXP alt, SEXP x) { R_set_altrep_data2(alt, x); }
 };
 
 // altrep R vector shadowing an primitive (int or double) Array.
@@ -141,6 +177,9 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
   static R_altrep_class_t class_t;
 
   using c_type = typename std::conditional<sexp_type == REALSXP, double, int>::type;
+  using Base::IsMaterialized;
+  using Base::Representation;
+  using Base::SetRepresentation;
 
   // Force materialization. After calling this, the data2 slot of the altrep
   // object contains a standard R vector with the same data, with
@@ -148,22 +187,22 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
   //
   // The Array remains available so that it can be used by Length(), Min(), etc ...
   static SEXP Materialize(SEXP alt) {
-    if (!Base::IsMaterialized(alt)) {
+    if (!IsMaterialized(alt)) {
       auto size = Base::Length(alt);
 
-      // create a standard R vector
+      // create an immutable standard R vector
       SEXP copy = PROTECT(Rf_allocVector(sexp_type, size));
+      MARK_NOT_MUTABLE(copy);
 
       // copy the data from the array, through Get_region
       Get_region(alt, 0, size, reinterpret_cast<c_type*>(DATAPTR(copy)));
 
       // store as data2, this is now considered materialized
-      R_set_altrep_data2(alt, copy);
-      MARK_NOT_MUTABLE(copy);
+      SetRepresentation(alt, copy);
 
       UNPROTECT(1);
     }
-    return R_altrep_data2(alt);
+    return Representation(alt);
   }
 
   // R calls this to get a pointer to the start of the vector data
@@ -171,14 +210,15 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
   static const void* Dataptr_or_null(SEXP alt) {
     // data2 has been created, and so the R sentinels are in place where the array has
     // nulls
-    if (Base::IsMaterialized(alt)) {
-      return DATAPTR_RO(R_altrep_data2(alt));
+    if (IsMaterialized(alt)) {
+      return DATAPTR_RO(Representation(alt));
     }
 
-    // the Array has no nulls, we can directly return the start of its data
-    const auto& array = GetArray(alt);
-    if (array->null_count() == 0) {
-      return reinterpret_cast<const void*>(array->data()->template GetValues<c_type>(1));
+    // there is only one chunk with no nulls, we can directly return the start of its data
+    auto chunked_array = GetChunkedArray(alt);
+    if (chunked_array->num_chunks() == 1 && chunked_array->null_count() == 0) {
+      return reinterpret_cast<const void*>(
+          chunked_array->chunk(0)->data()->template GetValues<c_type>(1));
     }
 
     // Otherwise: if the array has nulls and data2 has not been generated: give up
@@ -189,12 +229,12 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
   static void* Dataptr(SEXP alt, Rboolean writeable) {
     // If the object hasn't been materialized, and the array has no
     // nulls we can directly point to the array data.
-    if (!Base::IsMaterialized(alt)) {
-      const auto& array = GetArray(alt);
+    if (!IsMaterialized(alt)) {
+      const auto& chunked_array = GetChunkedArray(alt);
 
-      if (array->null_count() == 0) {
-        return reinterpret_cast<void*>(
-            const_cast<c_type*>(array->data()->template GetValues<c_type>(1)));
+      if (chunked_array->num_chunks() == 1 && chunked_array->null_count() == 0) {
+        return reinterpret_cast<void*>(const_cast<c_type*>(
+            chunked_array->chunk(0)->data()->template GetValues<c_type>(1)));
       }
     }
 
@@ -215,9 +255,12 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
 
   // The value at position i
   static c_type Elt(SEXP alt, R_xlen_t i) {
-    const auto& array = GetArray(alt);
-    return array->IsNull(i) ? cpp11::na<c_type>()
-                            : array->data()->template GetValues<c_type>(1)[i];
+    ArrayResolve resolve(GetChunkedArray(alt), i);
+    auto array = resolve.array_;
+    auto j = resolve.index_;
+
+    return array->IsNull(j) ? cpp11::na<c_type>()
+                            : array->data()->template GetValues<c_type>(1)[j];
   }
 
   // R calls this when it wants data from position `i` to `i + n` copied into `buf`
@@ -226,8 +269,8 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
   static R_xlen_t Get_region(SEXP alt, R_xlen_t i, R_xlen_t n, c_type* buf) {
     // If we have data2, we can just copy the region into buf
     // using the standard Get_region for this R type
-    if (Base::IsMaterialized(alt)) {
-      return Standard_Get_region<c_type>(R_altrep_data2(alt), i, n, buf);
+    if (IsMaterialized(alt)) {
+      return Standard_Get_region<c_type>(Representation(alt), i, n, buf);
     }
 
     // The vector was not materialized, aka we don't have data2
@@ -237,29 +280,35 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
     // array has nulls
     //
     // This only materialize the region, into buf. Not the entire vector.
-    auto slice = GetArray(alt)->Slice(i, n);
+    auto slice = GetChunkedArray(alt)->Slice(i, n);
     R_xlen_t ncopy = slice->length();
 
-    // first copy the data buffer
-    memcpy(buf, slice->data()->template GetValues<c_type>(1), ncopy * sizeof(c_type));
+    c_type* out = buf;
+    for (const auto& array : slice->chunks()) {
+      auto n_i = array->length();
 
-    // then set the R NA sentinels if needed
-    if (slice->null_count() > 0) {
-      internal::BitmapReader bitmap_reader(slice->null_bitmap()->data(), slice->offset(),
-                                           ncopy);
+      // first copy the data buffer
+      memcpy(out, array->data()->template GetValues<c_type>(1), n_i * sizeof(c_type));
 
-      for (R_xlen_t j = 0; j < ncopy; j++, bitmap_reader.Next()) {
-        if (bitmap_reader.IsNotSet()) {
-          buf[j] = cpp11::na<c_type>();
+      // then set the R NA sentinels if needed
+      if (array->null_count() > 0) {
+        internal::BitmapReader bitmap_reader(array->null_bitmap()->data(),
+                                             array->offset(), n_i);
+
+        for (R_xlen_t j = 0; j < n_i; j++, bitmap_reader.Next()) {
+          if (bitmap_reader.IsNotSet()) {
+            out[j] = cpp11::na<c_type>();
+          }
         }
       }
+
+      out += n_i;
     }
 
     return ncopy;
   }
 
-  static std::shared_ptr<arrow::compute::ScalarAggregateOptions> NaRmOptions(
-      const std::shared_ptr<Array>& array, bool na_rm) {
+  static std::shared_ptr<arrow::compute::ScalarAggregateOptions> NaRmOptions(bool na_rm) {
     auto options = std::make_shared<arrow::compute::ScalarAggregateOptions>(
         arrow::compute::ScalarAggregateOptions::Defaults());
     options->min_count = 0;
@@ -273,10 +322,10 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
     using scalar_type =
         typename std::conditional<sexp_type == INTSXP, Int32Scalar, DoubleScalar>::type;
 
-    const auto& array = GetArray(alt);
+    const auto& chunked_array = GetChunkedArray(alt);
     bool na_rm = narm == TRUE;
-    auto n = array->length();
-    auto null_count = array->null_count();
+    auto n = chunked_array->length();
+    auto null_count = chunked_array->null_count();
     if ((na_rm || n == 0) && null_count == n) {
       return Rf_ScalarReal(Min ? R_PosInf : R_NegInf);
     }
@@ -284,10 +333,10 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
       return cpp11::as_sexp(cpp11::na<data_type>());
     }
 
-    auto options = NaRmOptions(array, na_rm);
+    auto options = NaRmOptions(na_rm);
 
-    const auto& minmax =
-        ValueOrStop(arrow::compute::CallFunction("min_max", {array}, options.get()));
+    const auto& minmax = ValueOrStop(
+        arrow::compute::CallFunction("min_max", {chunked_array}, options.get()));
     const auto& minmax_scalar =
         internal::checked_cast<const StructScalar&>(*minmax.scalar());
 
@@ -303,17 +352,17 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
   static SEXP Sum(SEXP alt, Rboolean narm) {
     using data_type = typename std::conditional<sexp_type == REALSXP, double, int>::type;
 
-    const auto& array = GetArray(alt);
+    const auto& chunked_array = GetChunkedArray(alt);
     bool na_rm = narm == TRUE;
-    auto null_count = array->null_count();
+    auto null_count = chunked_array->null_count();
 
     if (!na_rm && null_count > 0) {
       return cpp11::as_sexp(cpp11::na<data_type>());
     }
-    auto options = NaRmOptions(array, na_rm);
+    auto options = NaRmOptions(na_rm);
 
     const auto& sum =
-        ValueOrStop(arrow::compute::CallFunction("sum", {array}, options.get()));
+        ValueOrStop(arrow::compute::CallFunction("sum", {chunked_array}, options.get()));
 
     if (sexp_type == INTSXP) {
       // When calling the "sum" function on an int32 array, we get an Int64 scalar
@@ -333,6 +382,306 @@ struct AltrepVectorPrimitive : public AltrepVectorBase<AltrepVectorPrimitive<sex
 template <int sexp_type>
 R_altrep_class_t AltrepVectorPrimitive<sexp_type>::class_t;
 
+struct AltrepFactor : public AltrepVectorBase<AltrepFactor> {
+  // singleton altrep class description
+  static R_altrep_class_t class_t;
+
+  using Base = AltrepVectorBase<AltrepFactor>;
+  using Base::IsMaterialized;
+
+  // redefining because data2 is a paired list with the representation as the
+  // first node: the CAR
+  static SEXP Representation(SEXP alt) { return CAR(R_altrep_data2(alt)); }
+
+  static void SetRepresentation(SEXP alt, SEXP x) { SETCAR(R_altrep_data2(alt), x); }
+
+  // The CADR(data2) is used to store the transposed arrays when unification is needed
+  // In that case we store a vector of Buffers
+  using BufferVector = std::vector<std::shared_ptr<Buffer>>;
+
+  static bool WasUnified(SEXP alt) { return !Rf_isNull(CADR(R_altrep_data2(alt))); }
+
+  static const std::shared_ptr<Buffer>& GetArrayTransposed(SEXP alt, int i) {
+    const auto& arrays = *Pointer<BufferVector>(CADR(R_altrep_data2(alt)));
+    return (*arrays)[i];
+  }
+
+  static SEXP Make(const std::shared_ptr<ChunkedArray>& chunked_array) {
+    // only dealing with dictionaries of strings
+    if (internal::checked_cast<const DictionaryArray&>(*chunked_array->chunk(0))
+            .dictionary()
+            ->type_id() != Type::STRING) {
+      return R_NilValue;
+    }
+
+    bool need_unification = DictionaryChunkArrayNeedUnification(chunked_array);
+
+    std::shared_ptr<Array> dictionary;
+    SEXP pointer_arrays_transpose;
+
+    if (need_unification) {
+      const auto& arr_type =
+          internal::checked_cast<const DictionaryType&>(*chunked_array->type());
+      std::unique_ptr<arrow::DictionaryUnifier> unifier_ =
+          ValueOrStop(DictionaryUnifier::Make(arr_type.value_type()));
+
+      size_t n_arrays = chunked_array->num_chunks();
+      BufferVector arrays_transpose(n_arrays);
+
+      for (size_t i = 0; i < n_arrays; i++) {
+        const auto& dict_i =
+            *internal::checked_cast<const DictionaryArray&>(*chunked_array->chunk(i))
+                 .dictionary();
+        StopIfNotOk(unifier_->Unify(dict_i, &arrays_transpose[i]));
+      }
+
+      std::shared_ptr<DataType> out_type;
+      StopIfNotOk(unifier_->GetResult(&out_type, &dictionary));
+
+      Pointer<BufferVector> ptr(
+          new std::shared_ptr<BufferVector>(new BufferVector(arrays_transpose)));
+      pointer_arrays_transpose = PROTECT(ptr);
+    } else {
+      // just use the first one
+      const auto& dict_array =
+          internal::checked_cast<const DictionaryArray&>(*chunked_array->chunk(0));
+      dictionary = dict_array.dictionary();
+
+      pointer_arrays_transpose = PROTECT(R_NilValue);
+    }
+
+    // the chunked array as data1
+    SEXP data1 =
+        PROTECT(Pointer<ChunkedArray>(new std::shared_ptr<ChunkedArray>(chunked_array)));
+
+    // a pairlist with the representation in the first node
+    SEXP data2 = PROTECT(Rf_list2(R_NilValue,  // representation, empty at first
+                                  pointer_arrays_transpose));
+
+    SEXP alt = PROTECT(R_new_altrep(class_t, data1, data2));
+    MARK_NOT_MUTABLE(alt);
+
+    // set factor attributes
+    Rf_setAttrib(alt, R_LevelsSymbol, Array__as_vector(dictionary));
+
+    if (internal::checked_cast<const DictionaryType&>(*chunked_array->type()).ordered()) {
+      Rf_classgets(alt, arrow::r::data::classes_ordered);
+    } else {
+      Rf_classgets(alt, arrow::r::data::classes_factor);
+    }
+
+    UNPROTECT(4);
+    return alt;
+  }
+
+  // TODO: this is similar to the primitive Materialize
+  static SEXP Materialize(SEXP alt) {
+    if (!IsMaterialized(alt)) {
+      auto size = Base::Length(alt);
+
+      // create a standard R vector
+      SEXP copy = PROTECT(Rf_allocVector(INTSXP, size));
+
+      // copy the data from the array, through Get_region
+      Get_region(alt, 0, size, reinterpret_cast<int*>(DATAPTR(copy)));
+
+      // store as data2, this is now considered materialized
+      SetRepresentation(alt, copy);
+      MARK_NOT_MUTABLE(copy);
+
+      UNPROTECT(1);
+    }
+    return Representation(alt);
+  }
+
+  static const void* Dataptr_or_null(SEXP alt) {
+    if (IsMaterialized(alt)) {
+      return DATAPTR_RO(Representation(alt));
+    }
+
+    return nullptr;
+  }
+
+  static void* Dataptr(SEXP alt, Rboolean writeable) { return DATAPTR(Materialize(alt)); }
+
+  static SEXP Duplicate(SEXP alt, Rboolean /* deep */) {
+    // the representation integer vector
+    SEXP dup = PROTECT(Rf_lazy_duplicate(Materialize(alt)));
+
+    // additional attributes from the altrep
+    SEXP atts = PROTECT(Rf_duplicate(ATTRIB(alt)));
+    SET_ATTRIB(dup, atts);
+
+    UNPROTECT(2);
+    return dup;
+  }
+
+  // The value at position i
+  static int Elt(SEXP alt, R_xlen_t i) {
+    if (Base::IsMaterialized(alt)) {
+      return INTEGER_ELT(Representation(alt), i);
+    }
+
+    ArrayResolve resolve(GetChunkedArray(alt), i);
+    auto array = resolve.array_;
+    auto j = resolve.index_;
+
+    if (!array->IsNull(j)) {
+      const auto& indices =
+          internal::checked_cast<const DictionaryArray&>(*array).indices();
+
+      if (WasUnified(alt)) {
+        const auto* transpose_data = reinterpret_cast<const int32_t*>(
+            GetArrayTransposed(alt, resolve.position_)->data());
+
+        switch (indices->type_id()) {
+          case Type::UINT8:
+            return transpose_data[indices->data()->GetValues<uint8_t>(1)[j]] + 1;
+          case Type::INT8:
+            return transpose_data[indices->data()->GetValues<int8_t>(1)[j]] + 1;
+          case Type::UINT16:
+            return transpose_data[indices->data()->GetValues<uint16_t>(1)[j]] + 1;
+          case Type::INT16:
+            return transpose_data[indices->data()->GetValues<int16_t>(1)[j]] + 1;
+          case Type::INT32:
+            return transpose_data[indices->data()->GetValues<int32_t>(1)[j]] + 1;
+          case Type::UINT32:
+            return transpose_data[indices->data()->GetValues<uint32_t>(1)[j]] + 1;
+          case Type::INT64:
+            return transpose_data[indices->data()->GetValues<int64_t>(1)[j]] + 1;
+          case Type::UINT64:
+            return transpose_data[indices->data()->GetValues<uint64_t>(1)[j]] + 1;
+          default:
+            break;
+        }
+      } else {
+        switch (indices->type_id()) {
+          case Type::UINT8:
+            return indices->data()->GetValues<uint8_t>(1)[j] + 1;
+          case Type::INT8:
+            return indices->data()->GetValues<int8_t>(1)[j] + 1;
+          case Type::UINT16:
+            return indices->data()->GetValues<uint16_t>(1)[j] + 1;
+          case Type::INT16:
+            return indices->data()->GetValues<int16_t>(1)[j] + 1;
+          case Type::INT32:
+            return indices->data()->GetValues<int32_t>(1)[j] + 1;
+          case Type::UINT32:
+            return indices->data()->GetValues<uint32_t>(1)[j] + 1;
+          case Type::INT64:
+            return indices->data()->GetValues<int64_t>(1)[j] + 1;
+          case Type::UINT64:
+            return indices->data()->GetValues<uint64_t>(1)[j] + 1;
+          default:
+            break;
+        }
+      }
+    }
+
+    // not reached
+    return NA_INTEGER;
+  }
+
+  static R_xlen_t Get_region(SEXP alt, R_xlen_t start, R_xlen_t n, int* buf) {
+    // If we have data2, we can just copy the region into buf
+    // using the standard Get_region for this R type
+    if (Base::IsMaterialized(alt)) {
+      return Standard_Get_region<int>(Representation(alt), start, n, buf);
+    }
+
+    auto chunked_array = GetChunkedArray(alt);
+
+    // get out if there is nothing to do
+    auto chunked_array_size = chunked_array->length();
+    if (start >= chunked_array_size) return 0;
+
+    auto slice = GetChunkedArray(alt)->Slice(start, n);
+
+    if (WasUnified(alt)) {
+      int j = 0;
+
+      // find out which is the first chunk of the chunk array
+      // that is present in the slice, because the main loop
+      // needs to refer to the correct transpose buffers
+      int64_t k = 0;
+      for (; j < chunked_array->num_chunks(); j++) {
+        auto nj = chunked_array->chunk(j)->length();
+        if (k + nj > start) {
+          break;
+        }
+
+        k += nj;
+      }
+
+      int* out = buf;
+      for (const auto& array : slice->chunks()) {
+        const auto& indices =
+            internal::checked_cast<const DictionaryArray&>(*array).indices();
+
+        // using the transpose data for this chunk
+        const auto* transpose_data =
+            reinterpret_cast<const int32_t*>(GetArrayTransposed(alt, j)->data());
+        auto transpose = [transpose_data](int x) { return transpose_data[x]; };
+
+        GetRegionDispatch(array, indices, transpose, out);
+
+        out += array->length();
+        j++;
+      }
+
+    } else {
+      // simpler case, identity transpose
+      auto transpose = [](int x) { return x; };
+
+      int* out = buf;
+      for (const auto& array : slice->chunks()) {
+        const auto& indices =
+            internal::checked_cast<const DictionaryArray&>(*array).indices();
+
+        GetRegionDispatch(array, indices, transpose, out);
+
+        out += array->length();
+      }
+    }
+
+    return slice->length();
+  }
+
+#define CALL_GET_REGION_TRANSPOSE(TYPE_CLASS)                                      \
+  case TYPE_CLASS##Type::type_id:                                                  \
+    GetRegionTranspose<TYPE_CLASS##Type>(array, indices,                           \
+                                         std::forward<Transpose>(transpose), out); \
+    break
+
+  template <typename Transpose>
+  static void GetRegionDispatch(const std::shared_ptr<Array>& array,
+                                const std::shared_ptr<Array>& indices,
+                                Transpose&& transpose, int* out) {
+    switch (indices->type_id()) {
+      ARROW_GENERATE_FOR_ALL_INTEGER_TYPES(CALL_GET_REGION_TRANSPOSE);
+      default:
+        break;
+    }
+  }
+
+  template <typename Type, typename Transpose>
+  static void GetRegionTranspose(const std::shared_ptr<Array>& array,
+                                 const std::shared_ptr<Array>& indices,
+                                 Transpose transpose, int* out) {
+    using index_type = typename Type::c_type;
+
+    VisitArraySpanInline<Type>(
+        *array->data(),
+        /*valid_func=*/[&](index_type index) { *out++ = transpose(index) + 1; },
+        /*null_func=*/[&]() { *out++ = cpp11::na<int>(); });
+  }
+
+  static SEXP Min(SEXP alt, Rboolean narm) { return nullptr; }
+  static SEXP Max(SEXP alt, Rboolean narm) { return nullptr; }
+  static SEXP Sum(SEXP alt, Rboolean narm) { return nullptr; }
+};
+R_altrep_class_t AltrepFactor::class_t;
+
 // Implementation for string arrays
 template <typename Type>
 struct AltrepVectorString : public AltrepVectorBase<AltrepVectorString<Type>> {
@@ -341,12 +690,13 @@ struct AltrepVectorString : public AltrepVectorBase<AltrepVectorString<Type>> {
   static R_altrep_class_t class_t;
   using StringArrayType = typename TypeTraits<Type>::ArrayType;
 
+  using Base::Representation;
+  using Base::SetRepresentation;
+
   // Helper class to convert to R strings
   struct RStringViewer {
-    explicit RStringViewer(const std::shared_ptr<Array>& array)
-        : array_(array),
-          string_array_(internal::checked_cast<const StringArrayType*>(array.get())),
-          strip_out_nuls_(GetBoolOption("arrow.skip_nul", false)),
+    RStringViewer()
+        : strip_out_nuls_(GetBoolOption("arrow.skip_nul", false)),
           nul_was_stripped_(false) {}
 
     // convert the i'th string of the Array to an R string (CHARSXP)
@@ -419,7 +769,12 @@ struct AltrepVectorString : public AltrepVectorBase<AltrepVectorString<Type>> {
       Rf_error(stripped_string_.c_str());
     }
 
-    const std::shared_ptr<Array>& array_;
+    void SetArray(const std::shared_ptr<Array>& array) {
+      array_ = array;
+      string_array_ = internal::checked_cast<const StringArrayType*>(array.get());
+    }
+
+    std::shared_ptr<Array> array_;
     const StringArrayType* string_array_;
     std::string stripped_string_;
     const bool strip_out_nuls_;
@@ -427,56 +782,45 @@ struct AltrepVectorString : public AltrepVectorBase<AltrepVectorString<Type>> {
     util::string_view view_;
   };
 
-  // Get a single string, as a CHARSXP SEXP,
-  // either from data2 or directly from the Array
+  // Get a single string, as a CHARSXP SEXP from data2.
+  // Materialize if not done so yet, given that it is
+  // likely that there will be another call from R if there is a call (e.g. unique()),
+  // and getting a string from Array is much more costly than from data2.
   static SEXP Elt(SEXP alt, R_xlen_t i) {
-    if (Base::IsMaterialized(alt)) {
-      return STRING_ELT(R_altrep_data2(alt), i);
+    if (!Base::IsMaterialized(alt)) {
+      Materialize(alt);
     }
-
-    BEGIN_CPP11
-
-    const auto& array = GetArray(alt);
-    RStringViewer r_string_viewer(array);
-
-    // r_string_viewer.Convert(i) might jump so it's wrapped
-    // in cpp11::unwind_protect() so that string_viewer
-    // can be properly destructed before the unwinding continues
-    SEXP s = NA_STRING;
-    cpp11::unwind_protect([&]() {
-      s = r_string_viewer.Convert(i);
-      if (r_string_viewer.nul_was_stripped()) {
-        cpp11::warning("Stripping '\\0' (nul) from character vector");
-      }
-    });
-    return s;
-
-    END_CPP11
+    return STRING_ELT(Representation(alt), i);
   }
 
   static void* Dataptr(SEXP alt, Rboolean writeable) { return DATAPTR(Materialize(alt)); }
 
   static SEXP Materialize(SEXP alt) {
     if (Base::IsMaterialized(alt)) {
-      return R_altrep_data2(alt);
+      return Representation(alt);
     }
 
     BEGIN_CPP11
 
-    const auto& array = GetArray(alt);
-    R_xlen_t n = array->length();
-    SEXP data2 = PROTECT(Rf_allocVector(STRSXP, n));
+    const auto& chunked_array = GetChunkedArray(alt);
+    SEXP data2 = PROTECT(Rf_allocVector(STRSXP, chunked_array->length()));
     MARK_NOT_MUTABLE(data2);
 
-    RStringViewer r_string_viewer(array);
+    RStringViewer r_string_viewer;
 
-    // r_string_viewer.Convert(i) might jump so we have to
+    // r_string_viewer.Convert() might jump so we have to
     // wrap it in unwind_protect() to:
     // - correctly destruct the C++ objects
     // - resume the unwinding
     cpp11::unwind_protect([&]() {
-      for (R_xlen_t i = 0; i < n; i++) {
-        SET_STRING_ELT(data2, i, r_string_viewer.Convert(i));
+      R_xlen_t i = 0;
+      for (const auto& array : chunked_array->chunks()) {
+        r_string_viewer.SetArray(array);
+
+        auto ni = array->length();
+        for (R_xlen_t j = 0; j < ni; j++, i++) {
+          SET_STRING_ELT(data2, i, r_string_viewer.Convert(j));
+        }
       }
 
       if (r_string_viewer.nul_was_stripped()) {
@@ -485,7 +829,7 @@ struct AltrepVectorString : public AltrepVectorBase<AltrepVectorString<Type>> {
     });
 
     // only set to data2 if all the values have been converted
-    R_set_altrep_data2(alt, data2);
+    SetRepresentation(alt, data2);
     UNPROTECT(1);  // data2
 
     return data2;
@@ -494,7 +838,7 @@ struct AltrepVectorString : public AltrepVectorBase<AltrepVectorString<Type>> {
   }
 
   static const void* Dataptr_or_null(SEXP alt) {
-    if (Base::IsMaterialized(alt)) return DATAPTR(R_altrep_data2(alt));
+    if (Base::IsMaterialized(alt)) return DATAPTR(Representation(alt));
 
     // otherwise give up
     return nullptr;
@@ -593,6 +937,7 @@ void InitAltStringClass(DllInfo* dll, const char* name) {
 void Init_Altrep_classes(DllInfo* dll) {
   InitAltRealClass<AltrepVectorPrimitive<REALSXP>>(dll, "arrow::array_dbl_vector");
   InitAltIntegerClass<AltrepVectorPrimitive<INTSXP>>(dll, "arrow::array_int_vector");
+  InitAltIntegerClass<AltrepFactor>(dll, "arrow::array_factor");
 
   InitAltStringClass<AltrepVectorString<StringType>>(dll, "arrow::array_string_vector");
   InitAltStringClass<AltrepVectorString<LargeStringType>>(
@@ -601,31 +946,31 @@ void Init_Altrep_classes(DllInfo* dll) {
 
 // return an altrep R vector that shadows the array if possible
 SEXP MakeAltrepVector(const std::shared_ptr<ChunkedArray>& chunked_array) {
-  // special case when there is only one array
-  if (chunked_array->num_chunks() == 1) {
-    const auto& array = chunked_array->chunk(0);
-    // using altrep if
-    // - the arrow.use_altrep is set to TRUE or unset (implicit TRUE)
-    // - the array has at least one element
-    if (arrow::r::GetBoolOption("arrow.use_altrep", true) && array->length() > 0) {
-      switch (array->type()->id()) {
-        case arrow::Type::DOUBLE:
-          return altrep::AltrepVectorPrimitive<REALSXP>::Make(array);
+  // using altrep if
+  // - the arrow.use_altrep is set to TRUE or unset (implicit TRUE)
+  // - the chunked array has at least one element
+  if (arrow::r::GetBoolOption("arrow.use_altrep", true) && chunked_array->length() > 0) {
+    switch (chunked_array->type()->id()) {
+      case arrow::Type::DOUBLE:
+        return altrep::AltrepVectorPrimitive<REALSXP>::Make(chunked_array);
 
-        case arrow::Type::INT32:
-          return altrep::AltrepVectorPrimitive<INTSXP>::Make(array);
+      case arrow::Type::INT32:
+        return altrep::AltrepVectorPrimitive<INTSXP>::Make(chunked_array);
 
-        case arrow::Type::STRING:
-          return altrep::AltrepVectorString<StringType>::Make(array);
+      case arrow::Type::STRING:
+        return altrep::AltrepVectorString<StringType>::Make(chunked_array);
 
-        case arrow::Type::LARGE_STRING:
-          return altrep::AltrepVectorString<LargeStringType>::Make(array);
+      case arrow::Type::LARGE_STRING:
+        return altrep::AltrepVectorString<LargeStringType>::Make(chunked_array);
 
-        default:
-          break;
-      }
+      case arrow::Type::DICTIONARY:
+        return altrep::AltrepFactor::Make(chunked_array);
+
+      default:
+        break;
     }
   }
+
   return R_NilValue;
 }
 
@@ -640,9 +985,9 @@ bool is_arrow_altrep(SEXP x) {
   return false;
 }
 
-std::shared_ptr<Array> vec_to_arrow_altrep_bypass(SEXP x) {
+std::shared_ptr<ChunkedArray> vec_to_arrow_altrep_bypass(SEXP x) {
   if (is_arrow_altrep(x)) {
-    return GetArray(x);
+    return GetChunkedArray(x);
   }
 
   return nullptr;
@@ -665,7 +1010,7 @@ SEXP MakeAltrepVector(const std::shared_ptr<ChunkedArray>& chunked_array) {
 
 bool is_arrow_altrep(SEXP) { return false; }
 
-std::shared_ptr<Array> vec_to_arrow_altrep_bypass(SEXP x) { return nullptr; }
+std::shared_ptr<ChunkedArray> vec_to_arrow_altrep_bypass(SEXP x) { return nullptr; }
 
 }  // namespace altrep
 }  // namespace r
@@ -676,15 +1021,5 @@ std::shared_ptr<Array> vec_to_arrow_altrep_bypass(SEXP x) { return nullptr; }
 // [[arrow::export]]
 void test_SET_STRING_ELT(SEXP s) { SET_STRING_ELT(s, 0, Rf_mkChar("forbidden")); }
 
-// [[arrow::export]]
-bool test_same_Array(SEXP x, SEXP y) {
-  auto* p_x = reinterpret_cast<std::shared_ptr<arrow::Array>*>(x);
-  auto* p_y = reinterpret_cast<std::shared_ptr<arrow::Array>*>(y);
-
-  return p_x->get() == p_y->get();
-}
-
 // [[arrow::export]]
 bool is_arrow_altrep(SEXP x) { return arrow::r::altrep::is_arrow_altrep(x); }
-
-#endif
