@@ -16,7 +16,6 @@
 // under the License.
 
 #include "./arrow_types.h"
-#if defined(ARROW_R_WITH_ARROW)
 
 #include <arrow/array/array_base.h>
 #include <arrow/table.h>
@@ -24,12 +23,14 @@
 #include <arrow/util/key_value_metadata.h>
 
 // [[arrow::export]]
-int Table__num_columns(const std::shared_ptr<arrow::Table>& x) {
-  return x->num_columns();
+r_vec_size Table__num_columns(const std::shared_ptr<arrow::Table>& x) {
+  return r_vec_size(x->num_columns());
 }
 
 // [[arrow::export]]
-int Table__num_rows(const std::shared_ptr<arrow::Table>& x) { return x->num_rows(); }
+r_vec_size Table__num_rows(const std::shared_ptr<arrow::Table>& x) {
+  return r_vec_size(x->num_rows());
+}
 
 // [[arrow::export]]
 std::shared_ptr<arrow::Schema> Table__schema(const std::shared_ptr<arrow::Table>& x) {
@@ -302,8 +303,8 @@ std::shared_ptr<arrow::Table> Table__from_record_batches(
 }
 
 // [[arrow::export]]
-int64_t Table__ReferencedBufferSize(const std::shared_ptr<arrow::Table>& table) {
-  return ValueOrStop(arrow::util::ReferencedBufferSize(*table));
+r_vec_size Table__ReferencedBufferSize(const std::shared_ptr<arrow::Table>& table) {
+  return r_vec_size(ValueOrStop(arrow::util::ReferencedBufferSize(*table)));
 }
 
 // [[arrow::export]]
@@ -313,5 +314,3 @@ std::shared_ptr<arrow::Table> Table__ConcatenateTables(
   options.unify_schemas = unify_schemas;
   return ValueOrStop(arrow::ConcatenateTables(tables, options));
 }
-
-#endif
