@@ -26,15 +26,15 @@
 #include "arrow/array/array_dict.h"
 #include "arrow/array/array_nested.h"
 #include "arrow/array/builder_dict.h"
-#include "arrow/compute/api_aggregate.h"
 #include "arrow/compute/api_scalar.h"
 #include "arrow/compute/api_vector.h"
 #include "arrow/compute/cast.h"
 #include "arrow/compute/exec/expression_internal.h"
+#include "arrow/compute/row/grouper.h"
 #include "arrow/dataset/dataset_internal.h"
 #include "arrow/filesystem/path_util.h"
 #include "arrow/scalar.h"
-#include "arrow/util/int_util_internal.h"
+#include "arrow/util/int_util_overflow.h"
 #include "arrow/util/logging.h"
 #include "arrow/util/make_unique.h"
 #include "arrow/util/string_view.h"
@@ -61,6 +61,18 @@ Result<std::string> SafeUriUnescape(util::string_view encoded) {
   }
   return decoded;
 }
+
+// Remove parts of the path after the last filename partition
+// separator. For example, 12_a_part-0.parquet will become 12_a_
+std::string StripNonPrefix(const std::string& path) {
+  std::string v;
+  auto non_prefix_index = path.rfind(kFilenamePartitionSep);
+  if (non_prefix_index != std::string::npos) {
+    v = path.substr(0, non_prefix_index + 1);
+  }
+  return v;
+}
+
 }  // namespace
 
 std::shared_ptr<Partitioning> Partitioning::Default() {
@@ -70,11 +82,15 @@ std::shared_ptr<Partitioning> Partitioning::Default() {
 
     std::string type_name() const override { return "default"; }
 
+    bool Equals(const Partitioning& other) const override {
+      return type_name() == other.type_name();
+    }
+
     Result<compute::Expression> Parse(const std::string& path) const override {
       return compute::literal(true);
     }
 
-    Result<std::string> Format(const compute::Expression& expr) const override {
+    Result<PartitionPathFormat> Format(const compute::Expression& expr) const override {
       return Status::NotImplemented("formatting paths from ", type_name(),
                                     " Partitioning");
     }
@@ -103,6 +119,28 @@ static Result<RecordBatchVector> ApplyGroupings(
   return out;
 }
 
+bool KeyValuePartitioning::Equals(const Partitioning& other) const {
+  if (this == &other) {
+    return true;
+  }
+  const auto& kv_partitioning = checked_cast<const KeyValuePartitioning&>(other);
+  const auto& other_dictionaries = kv_partitioning.dictionaries();
+  if (dictionaries_.size() != other_dictionaries.size()) {
+    return false;
+  }
+  int64_t idx = 0;
+  for (const auto& array : dictionaries_) {
+    const auto& other_array = other_dictionaries[idx++];
+    bool match = (array == nullptr && other_array == nullptr) ||
+                 (array && other_array && array->Equals(other_array));
+    if (!match) {
+      return false;
+    }
+  }
+  return options_.segment_encoding == kv_partitioning.options_.segment_encoding &&
+         Partitioning::Equals(other);
+}
+
 Result<Partitioning::PartitionedBatches> KeyValuePartitioning::Partition(
     const std::shared_ptr<RecordBatch>& batch) const {
   std::vector<int> key_indices;
@@ -123,20 +161,19 @@ Result<Partitioning::PartitionedBatches> KeyValuePartitioning::Partition(
     return PartitionedBatches{{batch}, {compute::literal(true)}};
   }
 
-  // assemble an ExecBatch of the key columns
-  compute::ExecBatch key_batch({}, batch->num_rows());
+  // assemble an ExecSpan of the key columns
+  compute::ExecSpan key_batch({}, batch->num_rows());
   for (int i : key_indices) {
-    key_batch.values.emplace_back(batch->column_data(i));
+    key_batch.values.emplace_back(ArraySpan(*batch->column_data(i)));
   }
 
-  ARROW_ASSIGN_OR_RAISE(auto grouper,
-                        compute::internal::Grouper::Make(key_batch.GetDescriptors()));
+  ARROW_ASSIGN_OR_RAISE(auto grouper, compute::Grouper::Make(key_batch.GetTypes()));
 
   ARROW_ASSIGN_OR_RAISE(Datum id_batch, grouper->Consume(key_batch));
 
   auto ids = id_batch.array_as<UInt32Array>();
-  ARROW_ASSIGN_OR_RAISE(auto groupings, compute::internal::Grouper::MakeGroupings(
-                                            *ids, grouper->num_groups()));
+  ARROW_ASSIGN_OR_RAISE(auto groupings,
+                        compute::Grouper::MakeGroupings(*ids, grouper->num_groups()));
 
   ARROW_ASSIGN_OR_RAISE(auto uniques, grouper->GetUniques());
   ArrayVector unique_arrays(num_keys);
@@ -251,7 +288,8 @@ Result<compute::Expression> KeyValuePartitioning::Parse(const std::string& path)
   return and_(std::move(expressions));
 }
 
-Result<std::string> KeyValuePartitioning::Format(const compute::Expression& expr) const {
+Result<PartitionPathFormat> KeyValuePartitioning::Format(
+    const compute::Expression& expr) const {
   ScalarVector values{static_cast<size_t>(schema_->num_fields()), nullptr};
 
   ARROW_ASSIGN_OR_RAISE(auto known_values, ExtractKnownFieldValues(expr));
@@ -292,19 +330,46 @@ Result<std::string> KeyValuePartitioning::Format(const compute::Expression& expr
   return FormatValues(values);
 }
 
-DirectoryPartitioning::DirectoryPartitioning(std::shared_ptr<Schema> schema,
-                                             ArrayVector dictionaries,
-                                             KeyValuePartitioningOptions options)
-    : KeyValuePartitioning(std::move(schema), std::move(dictionaries), options) {
-  util::InitializeUTF8();
+inline util::optional<int> NextValid(const ScalarVector& values, int first_null) {
+  auto it = std::find_if(values.begin() + first_null + 1, values.end(),
+                         [](const std::shared_ptr<Scalar>& v) { return v != nullptr; });
+
+  if (it == values.end()) {
+    return util::nullopt;
+  }
+
+  return static_cast<int>(it - values.begin());
 }
 
-Result<std::vector<KeyValuePartitioning::Key>> DirectoryPartitioning::ParseKeys(
-    const std::string& path) const {
-  std::vector<Key> keys;
+Result<std::vector<std::string>> KeyValuePartitioning::FormatPartitionSegments(
+    const ScalarVector& values) const {
+  std::vector<std::string> segments(static_cast<size_t>(schema_->num_fields()));
 
+  for (int i = 0; i < schema_->num_fields(); ++i) {
+    if (values[i] != nullptr && values[i]->is_valid) {
+      segments[i] = values[i]->ToString();
+      continue;
+    }
+
+    if (auto illegal_index = NextValid(values, i)) {
+      // XXX maybe we should just ignore keys provided after the first absent one?
+      return Status::Invalid("No partition key for ", schema_->field(i)->name(),
+                             " but a key was provided subsequently for ",
+                             schema_->field(*illegal_index)->name(), ".");
+    }
+
+    // if all subsequent keys are absent we'll just print the available keys
+    break;
+  }
+  return segments;
+}
+
+Result<std::vector<KeyValuePartitioning::Key>>
+KeyValuePartitioning::ParsePartitionSegments(
+    const std::vector<std::string>& segments) const {
   int i = 0;
-  for (auto&& segment : fs::internal::SplitAbstractPath(path)) {
+  std::vector<Key> keys;
+  for (auto&& segment : segments) {
     if (i >= schema_->num_fields()) break;
 
     switch (options_.segment_encoding) {
@@ -325,43 +390,56 @@ Result<std::vector<KeyValuePartitioning::Key>> DirectoryPartitioning::ParseKeys(
                                       options_.segment_encoding);
     }
   }
-
   return keys;
 }
 
-inline util::optional<int> NextValid(const ScalarVector& values, int first_null) {
-  auto it = std::find_if(values.begin() + first_null + 1, values.end(),
-                         [](const std::shared_ptr<Scalar>& v) { return v != nullptr; });
-
-  if (it == values.end()) {
-    return util::nullopt;
-  }
-
-  return static_cast<int>(it - values.begin());
+DirectoryPartitioning::DirectoryPartitioning(std::shared_ptr<Schema> schema,
+                                             ArrayVector dictionaries,
+                                             KeyValuePartitioningOptions options)
+    : KeyValuePartitioning(std::move(schema), std::move(dictionaries), options) {
+  util::InitializeUTF8();
 }
 
-Result<std::string> DirectoryPartitioning::FormatValues(
+Result<std::vector<KeyValuePartitioning::Key>> DirectoryPartitioning::ParseKeys(
+    const std::string& path) const {
+  std::vector<std::string> segments =
+      fs::internal::SplitAbstractPath(fs::internal::GetAbstractPathParent(path).first);
+  return ParsePartitionSegments(segments);
+}
+
+bool DirectoryPartitioning::Equals(const Partitioning& other) const {
+  return type_name() == other.type_name() && KeyValuePartitioning::Equals(other);
+}
+
+FilenamePartitioning::FilenamePartitioning(std::shared_ptr<Schema> schema,
+                                           ArrayVector dictionaries,
+                                           KeyValuePartitioningOptions options)
+    : KeyValuePartitioning(std::move(schema), std::move(dictionaries), options) {
+  util::InitializeUTF8();
+}
+
+Result<std::vector<KeyValuePartitioning::Key>> FilenamePartitioning::ParseKeys(
+    const std::string& path) const {
+  std::vector<std::string> segments = fs::internal::SplitAbstractPath(
+      StripNonPrefix(fs::internal::GetAbstractPathParent(path).second),
+      kFilenamePartitionSep);
+  return ParsePartitionSegments(segments);
+}
+
+Result<PartitionPathFormat> DirectoryPartitioning::FormatValues(
     const ScalarVector& values) const {
-  std::vector<std::string> segments(static_cast<size_t>(schema_->num_fields()));
+  std::vector<std::string> segments;
+  ARROW_ASSIGN_OR_RAISE(segments, FormatPartitionSegments(values));
+  return PartitionPathFormat{fs::internal::JoinAbstractPath(std::move(segments)), ""};
+}
 
-  for (int i = 0; i < schema_->num_fields(); ++i) {
-    if (values[i] != nullptr && values[i]->is_valid) {
-      segments[i] = values[i]->ToString();
-      continue;
-    }
-
-    if (auto illegal_index = NextValid(values, i)) {
-      // XXX maybe we should just ignore keys provided after the first absent one?
-      return Status::Invalid("No partition key for ", schema_->field(i)->name(),
-                             " but a key was provided subsequently for ",
-                             schema_->field(*illegal_index)->name(), ".");
-    }
-
-    // if all subsequent keys are absent we'll just print the available keys
-    break;
-  }
-
-  return fs::internal::JoinAbstractPath(std::move(segments));
+Result<PartitionPathFormat> FilenamePartitioning::FormatValues(
+    const ScalarVector& values) const {
+  std::vector<std::string> segments;
+  ARROW_ASSIGN_OR_RAISE(segments, FormatPartitionSegments(values));
+  return PartitionPathFormat{
+      "", fs::internal::JoinAbstractPath(std::move(segments), kFilenamePartitionSep) +
+              kFilenamePartitionSep};
 }
 
 KeyValuePartitioningOptions PartitioningFactoryOptions::AsPartitioningOptions() const {
@@ -488,6 +566,33 @@ class KeyValuePartitioningFactory : public PartitioningFactory {
                                                                utf8());
   }
 
+  Status InspectPartitionSegments(std::vector<std::string> segments,
+                                  const std::vector<std::string>& field_names) {
+    size_t field_index = 0;
+    for (auto&& segment : segments) {
+      if (field_index == field_names.size()) break;
+
+      switch (options_.segment_encoding) {
+        case SegmentEncoding::None: {
+          if (ARROW_PREDICT_FALSE(!util::ValidateUTF8(segment))) {
+            return Status::Invalid("Partition segment was not valid UTF-8: ", segment);
+          }
+          RETURN_NOT_OK(InsertRepr(static_cast<int>(field_index++), segment));
+          break;
+        }
+        case SegmentEncoding::Uri: {
+          ARROW_ASSIGN_OR_RAISE(auto decoded, SafeUriUnescape(segment));
+          RETURN_NOT_OK(InsertRepr(static_cast<int>(field_index++), decoded));
+          break;
+        }
+        default:
+          return Status::NotImplemented("Unknown segment encoding: ",
+                                        options_.segment_encoding);
+      }
+    }
+    return Status::OK();
+  }
+
   PartitioningFactoryOptions options_;
   ArrayVector dictionaries_;
   std::unordered_map<std::string, int> name_to_index_;
@@ -508,30 +613,10 @@ class DirectoryPartitioningFactory : public KeyValuePartitioningFactory {
   Result<std::shared_ptr<Schema>> Inspect(
       const std::vector<std::string>& paths) override {
     for (auto path : paths) {
-      size_t field_index = 0;
-      for (auto&& segment : fs::internal::SplitAbstractPath(path)) {
-        if (field_index == field_names_.size()) break;
-
-        switch (options_.segment_encoding) {
-          case SegmentEncoding::None: {
-            if (ARROW_PREDICT_FALSE(!util::ValidateUTF8(segment))) {
-              return Status::Invalid("Partition segment was not valid UTF-8: ", segment);
-            }
-            RETURN_NOT_OK(InsertRepr(static_cast<int>(field_index++), segment));
-            break;
-          }
-          case SegmentEncoding::Uri: {
-            ARROW_ASSIGN_OR_RAISE(auto decoded, SafeUriUnescape(segment));
-            RETURN_NOT_OK(InsertRepr(static_cast<int>(field_index++), decoded));
-            break;
-          }
-          default:
-            return Status::NotImplemented("Unknown segment encoding: ",
-                                          options_.segment_encoding);
-        }
-      }
+      std::vector<std::string> segments;
+      segments = fs::internal::SplitAbstractPath(path);
+      RETURN_NOT_OK(InspectPartitionSegments(segments, field_names_));
     }
-
     return DoInspect();
   }
 
@@ -561,6 +646,54 @@ class DirectoryPartitioningFactory : public KeyValuePartitioningFactory {
   std::vector<std::string> field_names_;
 };
 
+class FilenamePartitioningFactory : public KeyValuePartitioningFactory {
+ public:
+  FilenamePartitioningFactory(std::vector<std::string> field_names,
+                              PartitioningFactoryOptions options)
+      : KeyValuePartitioningFactory(options), field_names_(std::move(field_names)) {
+    Reset();
+    util::InitializeUTF8();
+  }
+
+  std::string type_name() const override { return "filename"; }
+
+  Result<std::shared_ptr<Schema>> Inspect(
+      const std::vector<std::string>& paths) override {
+    for (const auto& path : paths) {
+      std::vector<std::string> segments;
+      segments =
+          fs::internal::SplitAbstractPath(StripNonPrefix(path), kFilenamePartitionSep);
+      RETURN_NOT_OK(InspectPartitionSegments(segments, field_names_));
+    }
+    return DoInspect();
+  }
+
+  Result<std::shared_ptr<Partitioning>> Finish(
+      const std::shared_ptr<Schema>& schema) const override {
+    for (FieldRef ref : field_names_) {
+      // ensure all of field_names_ are present in schema
+      RETURN_NOT_OK(ref.FindOne(*schema).status());
+    }
+
+    // drop fields which aren't in field_names_
+    auto out_schema = SchemaFromColumnNames(schema, field_names_);
+
+    return std::make_shared<FilenamePartitioning>(std::move(out_schema), dictionaries_,
+                                                  options_.AsPartitioningOptions());
+  }
+
+ private:
+  void Reset() override {
+    KeyValuePartitioningFactory::Reset();
+
+    for (const auto& name : field_names_) {
+      GetOrInsertField(name);
+    }
+  }
+
+  std::vector<std::string> field_names_;
+};
+
 }  // namespace
 
 std::shared_ptr<PartitioningFactory> DirectoryPartitioning::MakeFactory(
@@ -569,6 +702,19 @@ std::shared_ptr<PartitioningFactory> DirectoryPartitioning::MakeFactory(
       new DirectoryPartitioningFactory(std::move(field_names), options));
 }
 
+std::shared_ptr<PartitioningFactory> FilenamePartitioning::MakeFactory(
+    std::vector<std::string> field_names, PartitioningFactoryOptions options) {
+  return std::shared_ptr<PartitioningFactory>(
+      new FilenamePartitioningFactory(std::move(field_names), options));
+}
+
+bool FilenamePartitioning::Equals(const Partitioning& other) const {
+  if (type_name() != other.type_name()) {
+    return false;
+  }
+  return KeyValuePartitioning::Equals(other);
+}
+
 Result<util::optional<KeyValuePartitioning::Key>> HivePartitioning::ParseKey(
     const std::string& segment, const HivePartitioningOptions& options) {
   auto name_end = string_view(segment).find_first_of('=');
@@ -613,7 +759,8 @@ Result<std::vector<KeyValuePartitioning::Key>> HivePartitioning::ParseKeys(
     const std::string& path) const {
   std::vector<Key> keys;
 
-  for (const auto& segment : fs::internal::SplitAbstractPath(path)) {
+  for (const auto& segment :
+       fs::internal::SplitAbstractPath(fs::internal::GetAbstractPathParent(path).first)) {
     ARROW_ASSIGN_OR_RAISE(auto maybe_key, ParseKey(segment, hive_options_));
     if (auto key = maybe_key) {
       keys.push_back(std::move(*key));
@@ -623,7 +770,8 @@ Result<std::vector<KeyValuePartitioning::Key>> HivePartitioning::ParseKeys(
   return keys;
 }
 
-Result<std::string> HivePartitioning::FormatValues(const ScalarVector& values) const {
+Result<PartitionPathFormat> HivePartitioning::FormatValues(
+    const ScalarVector& values) const {
   std::vector<std::string> segments(static_cast<size_t>(schema_->num_fields()));
 
   for (int i = 0; i < schema_->num_fields(); ++i) {
@@ -640,7 +788,20 @@ Result<std::string> HivePartitioning::FormatValues(const ScalarVector& values) c
     }
   }
 
-  return fs::internal::JoinAbstractPath(std::move(segments));
+  return PartitionPathFormat{fs::internal::JoinAbstractPath(std::move(segments)), ""};
+}
+
+bool HivePartitioning::Equals(const Partitioning& other) const {
+  if (this == &other) {
+    return true;
+  }
+  if (type_name() != other.type_name()) {
+    return false;
+  }
+  const auto& hive_part = ::arrow::internal::checked_cast<const HivePartitioning&>(other);
+  return null_fallback() == hive_part.null_fallback() &&
+         options().null_fallback == hive_part.options().null_fallback &&
+         KeyValuePartitioning::Equals(other);
 }
 
 class HivePartitioningFactory : public KeyValuePartitioningFactory {
@@ -695,9 +856,14 @@ std::shared_ptr<PartitioningFactory> HivePartitioning::MakeFactory(
   return std::shared_ptr<PartitioningFactory>(new HivePartitioningFactory(options));
 }
 
-std::string StripPrefixAndFilename(const std::string& path, const std::string& prefix) {
+std::string StripPrefix(const std::string& path, const std::string& prefix) {
   auto maybe_base_less = fs::internal::RemoveAncestor(prefix, path);
   auto base_less = maybe_base_less ? std::string(*maybe_base_less) : path;
+  return base_less;
+}
+
+std::string StripPrefixAndFilename(const std::string& path, const std::string& prefix) {
+  auto base_less = StripPrefix(path, prefix);
   auto basename_filename = fs::internal::GetAbstractPathParent(base_less);
   return basename_filename.first;
 }
@@ -727,7 +893,6 @@ Result<std::shared_ptr<Schema>> PartitioningOrFactory::GetOrInferSchema(
   if (auto part = partitioning()) {
     return part->schema();
   }
-
   return factory()->Inspect(paths);
 }
 
